package io.arex.foundation.listener;

import io.arex.agent.bootstrap.cache.TimeCache;
import io.arex.foundation.config.ConfigManager;
import io.arex.foundation.context.ContextManager;
import io.arex.foundation.model.Constants;
import io.arex.foundation.model.DynamicClassMocker;
import io.arex.foundation.util.LogUtil;
import org.apache.commons.lang3.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CaseInitializer {
    private static final Logger LOGGER = LoggerFactory.getLogger(CaseInitializer.class);

    public static void initialize(String caseId){
        initContext(caseId);
        initClock();
    }

    public static void initContext(String caseId){
        ContextManager.currentContext(true, caseId);
    }

    public static void initClock(){
        try {
            if (ConfigManager.INSTANCE.startTimeMachine() && ContextManager.needReplay()) {
                DynamicClassMocker mocker = new DynamicClassMocker(Constants.CLOCK_CLASS, Constants.CLOCK_METHOD, null);
                Object result = mocker.replay();
                long millis = NumberUtils.toLong(String.valueOf(result), 0);
                if (millis > 0) {
                    TimeCache.put(TimeCache.PREFIX_MILLIS, millis);
                    TimeCache.put(TimeCache.PREFIX_NANO, System.nanoTime());
                }
            } else if (ContextManager.needRecord()) {
                DynamicClassMocker mocker = new DynamicClassMocker(Constants.CLOCK_CLASS, Constants.CLOCK_METHOD,
                        null, String.valueOf(System.currentTimeMillis()), Long.class.getName());
                mocker.record();
            }
        } catch (Throwable e) {
            LOGGER.warn(LogUtil.buildTitle("time.machine.init"), e);
        }
    }

    public static void release(){
        TimeCache.remove();
        releaseContext();
    }

    public static void releaseContext(){
        ContextManager.remove();
    }
}
